#include "OgreVisibilityOctreeSceneManager.h"
#include <OgreMath.h>
#include <OgreIteratorWrappers.h>
#include <OgreRenderSystem.h>
#include <OgreCamera.h>

#include <windows.h>

namespace Ogre {

//-----------------------------------------------------------------------
VisibilityOctreeSceneManager::VisibilityOctreeSceneManager(
	GtpVisibility::VisibilityManager *visManager): mVisibilityManager(visManager)
{
	mHierarchyInterface = 
		new OctreeHierarchyInterface(this, mDestRenderSystem);

	//mDisplayNodes = true;
	//mShowBoundingBoxes = true;
	//mShowBoxes = true;
	//mMaxDepth = 20;
}
//-----------------------------------------------------------------------
VisibilityOctreeSceneManager::~VisibilityOctreeSceneManager()
{
	delete mHierarchyInterface;
}
//-----------------------------------------------------------------------
void VisibilityOctreeSceneManager::_renderVisibleObjects()
{
	//-- render background
	clearSpecialCaseRenderQueues();
	addSpecialCaseRenderQueue(RENDER_QUEUE_BACKGROUND);
	addSpecialCaseRenderQueue(RENDER_QUEUE_SKIES_EARLY);

	setSpecialCaseRenderQueueMode(SceneManager::SCRQM_INCLUDE);
	SceneManager::_renderVisibleObjects( );
#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	_deleteRenderedQueueGroups();
#endif

	//-- render visible objects (i.e., all but overlay)
	clearSpecialCaseRenderQueues();
	addSpecialCaseRenderQueue(RENDER_QUEUE_OVERLAY);
	setSpecialCaseRenderQueueMode(SceneManager::SCRQM_EXCLUDE);

	mHierarchyInterface->InitFrame(mSceneRoot, mCameraInProgress);
	mVisibilityManager->ApplyVisibilityCulling();

#ifdef GTP_VISIBILITY_MODIFIED_OGRE
	_deleteRenderedQueueGroups();
#endif

	//-- render rest, e.g., overlay
	clearSpecialCaseRenderQueues();
	SceneManager::_renderVisibleObjects();
	//_deleteRenderedQueueGroups();
}
//-----------------------------------------------------------------------
void VisibilityOctreeSceneManager::_findVisibleObjects(Camera* cam, bool onlyShadowCasters)
{
	// must be empty because objects are found and rendered in an interleaved fashion
	// in _renderVisibibleObjects 
}
//-----------------------------------------------------------------------
void VisibilityOctreeSceneManager::_updateSceneGraph(Camera* cam)
{
	//mOcclusionOctreeHierarchyInterface->setNumSceneNodes((int)mSceneNodes.size());
	mVisibilityManager->GetCullingManager()->SetHierarchyInterface(mHierarchyInterface);
	mHierarchyInterface->SetRenderSystem(mDestRenderSystem);

#ifdef GTP_VISIBILITY_MODIFIED_OGRE
    mHierarchyInterface->SetNumOctreeNodes(mNumOctreeNodes);
#endif
	OctreeSceneManager::_updateSceneGraph(cam);
}
//-----------------------------------------------------------------------
bool VisibilityOctreeSceneManager::setOption(const String & key, const void * val)
{
	return /*mVisibilitySceneTraverser->setOption( key, val) || */OctreeSceneManager::setOption( key, val );
}
//-----------------------------------------------------------------------
bool VisibilityOctreeSceneManager::getOption(const String & key, void *val)
{
	return /*mVisibilitySceneTraverser->getOption( key, val) || */OctreeSceneManager::getOption(key, val);
}
//-----------------------------------------------------------------------
bool VisibilityOctreeSceneManager::getOptionValues(const String & key, StringVector  &refValueList)
{
	return OctreeSceneManager::getOptionValues( key, refValueList );
}
//-----------------------------------------------------------------------
bool VisibilityOctreeSceneManager::getOptionKeys(StringVector & refKeys)
{
	return /*mVisibilitySceneTraverser->getOptionKeys ( refKeys ) || */OctreeSceneManager::getOptionKeys(refKeys);
}
//-----------------------------------------------------------------------
void VisibilityOctreeSceneManager::setVisibilityManager(GtpVisibility::VisibilityManager *visManager)
{
    mVisibilityManager = visManager;
}
//-----------------------------------------------------------------------
GtpVisibility::VisibilityManager *VisibilityOctreeSceneManager::getVisibilityManager()
{
	return mVisibilityManager;
}

}   // namespace Ogre